package no.ntnu.master.form;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.scenes.scene2d.utils.ScissorStack;
import com.badlogic.gdx.utils.viewport.ScreenViewport;
import com.badlogic.gdx.utils.viewport.Viewport;

import java.text.DecimalFormat;

import no.ntnu.master.objects.LineSegment;
import no.ntnu.master.utils.AssetLoader;
import no.ntnu.master.utils.Enums;

/**
 * Created by B on 22.12.2015.
 * Renderer for the form portion of the application
 */
public class FormRenderer {

    private OrthographicCamera cam;
    private Viewport viewport;

    private ShapeRenderer shapeRenderer = new ShapeRenderer();

    private FormWorld world;
    private SpriteBatch batch;

    private Texture lineTexture;

    private Rectangle scissor1;
    private Rectangle clippingBounds;

    private final String FINISHED_TEXT = "Click screen\nto view results";

    private DecimalFormat df = new DecimalFormat("#.00");

    public FormRenderer(FormWorld world, SpriteBatch batch) {
        cam = new OrthographicCamera();
        viewport = new ScreenViewport(cam);
        viewport.apply();
        cam.position.set(cam.viewportWidth / 2f, cam.viewportHeight / 2f, 0f);

        this.world = world;
        this.batch = batch;

        scissor1 = new Rectangle();
        clippingBounds = new Rectangle();

        lineTexture = AssetLoader.lineTexture;
    }

    public void render(float delta) {

        Vector2 tempX = new Vector2(world.rectLeft.x, 0);
        Vector2 tempY = new Vector2(0, world.rectLeft.y);
        clippingBounds.set(viewport.unproject(tempX).x, -(viewport.unproject(tempY).y),
                world.rectLeft.width + world.patchGap + world.rectRight.width, world.rectLeft.height);

        Gdx.gl.glClearColor(0f, 0f, 0f, 1f);
        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);

        shapeRenderer.begin(ShapeRenderer.ShapeType.Line);
        shapeRenderer.setColor(Color.WHITE);
        shapeRenderer.rect(world.rectLeft.x, world.rectLeft.y,
                world.rectLeft.width, world.rectLeft.height);
        shapeRenderer.rect(world.rectRight.x, world.rectRight.y,
                world.rectRight.width, world.rectRight.height);
        shapeRenderer.end();

        ScissorStack.calculateScissors(cam, batch.getTransformMatrix(), clippingBounds, scissor1);
        ScissorStack.pushScissors(scissor1);

        if (world.getState() == Enums.WorldStates.FINISHED) {
            finished();
        }

        batch.begin();

        if (world.getState() == Enums.WorldStates.RUNNING) {
            for (int i = 0; i < world.segments.size; i++) {
                LineSegment ls = world.segments.get(i);
                batch.draw(lineTexture,                                         //texture
                        ls.getX()-ls.getLength()/2, ls.getY()-ls.getHeight()/2, //x, y
                        ls.getLength()/2, ls.getHeight()/2,                     //originX, originY
                        ls.getLength(), ls.getHeight(),                         //width, height
                        1f, 1f,                                                 //scaleX, scaleY
                        ls.getAngle(),                                          //rotation
                        0, 0,                                                   //scrX, srcY
                        200,1,                                                  //scrWidth, srcHeight
                        false,false                                             //flipX, flipY
                );
            }
        }
//        int fps = Gdx.graphics.getFramesPerSecond();
//        AssetLoader.font.draw(batch, "FPS: " + fps + "\n"
//                        /*+ world.getCoherentPatch() + "\n"*/
//                        + "C: " + df.format(world.coherencePercent) + "%" + "\n"
//                        + "Correct side:" + "\n"
//                        + world.getCoherentPatch() + "\n"
//                        + "Correct: " + world.correctAnswerCounter + "\n"
//                        + "Wrong: " + world.wrongAnswerCounter + "\n"
//                        + "Reversals: " + world.reversalCounter,
//                (world.rectLeft.x + world.rectLeft.width) + 10f, (Settings.getScreenHeightPixels() / 2f) + 100);

        batch.end();
        batch.flush();
        ScissorStack.popScissors();

        //For a smooth vanishing of lines in the center of the screen
        shapeRenderer.begin(ShapeRenderer.ShapeType.Filled);
        shapeRenderer.setColor(Color.BLACK);
        shapeRenderer.rect((world.rectLeft.x + world.rectLeft.width)+0.5f, world.rectLeft.y,
                (world.patchGap)-1f, world.rectLeft.height);
        shapeRenderer.end();
    }

    private void finished() {
        batch.begin();
        AssetLoader.font.draw(batch, FINISHED_TEXT, world.rectLeft.x+FINISHED_TEXT.length(),
                Gdx.graphics.getHeight() / 2);
        AssetLoader.font.draw(batch, FINISHED_TEXT, world.rectRight.x+FINISHED_TEXT.length(),
                Gdx.graphics.getHeight() / 2);
        batch.end();
    }

    public void resize(int width, int height) {
        viewport.update(width, height);
        cam.position.set(cam.viewportWidth / 2f, cam.viewportHeight / 2f, 0f);
    }
}
