package no.ntnu.master.motion;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.scenes.scene2d.utils.ScissorStack;
import com.badlogic.gdx.utils.viewport.ScreenViewport;
import com.badlogic.gdx.utils.viewport.Viewport;

import java.text.DecimalFormat;

import no.ntnu.master.objects.Dot;
import no.ntnu.master.utils.AssetLoader;

/**
 * Created by B on 16.12.2015.
 * Renderer for the motion part of the application
 */
public class MotionRenderer {

    private OrthographicCamera cam;
    private Viewport viewport;

    private SpriteBatch batch;
    private MotionWorld world;

    private ShapeRenderer shapeRenderer = new ShapeRenderer();

    private Rectangle scissor1;
//    private Rectangle leftClippingBounds;
//    private Rectangle rightClippingBounds;
    private Rectangle clippingBounds;

    private Texture dotTexture;

    private final String FINISHED_TEXT = "Click screen\nto view results";

    private DecimalFormat df = new DecimalFormat("#.00");

    public MotionRenderer(MotionWorld world, SpriteBatch batch) {
        cam = new OrthographicCamera();
        viewport = new ScreenViewport(cam);
        viewport.apply();
        cam.position.set(cam.viewportWidth / 2f, cam.viewportHeight / 2f, 0f);



        this.batch = batch;
        this.world = world;

        scissor1 = new Rectangle();
        clippingBounds = new Rectangle();

        this.dotTexture = AssetLoader.dotTexture;
    }

    public void render(float delta) {

        Vector2 tempX = new Vector2(world.rectLeft.x, 0);
        Vector2 tempY = new Vector2(0, world.rectLeft.y);
        clippingBounds.set(viewport.unproject(tempX).x, -(viewport.unproject(tempY).y), world.rectLeft.width + world.patchGap + world.rectRight.width, world.rectLeft.height);

        Gdx.gl.glClearColor(0f, 0f, 0f, 1f);
        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);

        shapeRenderer.begin(ShapeRenderer.ShapeType.Line);
        shapeRenderer.setColor(Color.WHITE);
        shapeRenderer.rect(world.rectLeft.x, world.rectLeft.y, world.rectLeft.width, world.rectLeft.height);
        shapeRenderer.rect(world.rectRight.x, world.rectRight.y, world.rectRight.width, world.rectRight.height);

        shapeRenderer.end();

        ScissorStack.calculateScissors(cam, batch.getTransformMatrix(), clippingBounds, scissor1);
        ScissorStack.pushScissors(scissor1);

        switch (world.getState()) {
            case FINISHED: finished(); break;
            case PAUSED: break;
            case RUNNING: renderDots() ;break;
        }

//        batch.begin();
//
//        int fps = Gdx.graphics.getFramesPerSecond();
//        AssetLoader.font.draw(batch, "FPS: " + fps + "\n"
//                        /*+ world.getCoherentPatch() + "\n"*/
//                        + "C: " + df.format(world.coherencePercent) + "%" + "\n"
//                        + "Correct side:" + "\n"
//                        + world.getCoherentPatch() + "\n"
//                        + "Correct: " + world.correctAnswerCounter + "\n"
//                        + "Wrong: " + world.wrongAnswerCounter + "\n"
//                        + "Reversals: " + world.reversalCounter,
//                (world.rectLeft.x+world.rectLeft.width)+10f, (Settings.getScreenHeightPixels() / 2f)+100);
//        batch.end();
        batch.flush();
        ScissorStack.popScissors();
    }

    private void finished() {
        batch.begin();
        AssetLoader.font.draw(batch, FINISHED_TEXT, world.rectLeft.x+FINISHED_TEXT.length(), Gdx.graphics.getHeight() / 2);
        AssetLoader.font.draw(batch, FINISHED_TEXT, world.rectRight.x+FINISHED_TEXT.length(), Gdx.graphics.getHeight() / 2);
        batch.end();
    }


    private void renderDots() {

        batch.begin();

        for (int i = 0; i<world.dotsLeft.size();i++) {
            Dot dot = world.dotsLeft.get(i);
            if (dot.getX() < (world.rectLeft.x+world.rectLeft.width)+dot.getRadius() && dot.getAliveTimer() > 0f)
            {
                batch.draw(dotTexture,                              //texture
                        dot.getPosition().x, dot.getPosition().y,   //x, y
                        dot.getRadius(), dot.getRadius(),           //originX, originY
                        dot.getRadius()*2, dot.getRadius()*2,       //width, height
                        1f, 1f,                                     //scaleX, scaleY
                        0,                                          //rotation
                        0, 0,                                       //scrX, srcY
                        1658,1658,                                  //scrWidth, srcHeight
                        false,false                                 //flipX, flipY
                );
            }
        }

        for (int i = 0; i<world.dotsRight.size();i++) {
            Dot dot = world.dotsRight.get(i);
            if (dot.getX() > world.rectRight.x-(dot.getRadius()*2) && dot.getAliveTimer() > 0f)
            {
                batch.draw(dotTexture,                              //texture
                        dot.getPosition().x, dot.getPosition().y,   //x, y
                        dot.getRadius(), dot.getRadius(),           //originX, originY
                        dot.getRadius()*2, dot.getRadius()*2,       //width, height
                        1f, 1f,                                     //scaleX, scaleY
                        0,                                         //rotation
                        0, 0,                                       //scrX, srcY
                        1658,1658,                                  //scrWidth, srcHeight
                        false,false                                 //flipX, flipY
                );
            }
        }





//        int fps = Gdx.graphics.getFramesPerSecond();
//        AssetLoader.font.draw(batch, "" + fps + "\n"
//                                    + world.getCoherentPatch() + "\n"
//                                    + "C: " + world.coherencePercent + "\n"
//                                    + "R: " + "p" + "\n" + "W: " + "p",
//                (Settings.getScreenWidthPixels() / 2f)-60, Settings.getScreenHeightPixels() / 2f);


        batch.end();

        //For a smooth vanishing of dots in the center of the screen
        shapeRenderer.begin(ShapeRenderer.ShapeType.Filled);
        shapeRenderer.setColor(Color.BLACK);
        shapeRenderer.rect((world.rectLeft.x + world.rectLeft.width) + 0.5f, world.rectLeft.y,
                (world.patchGap) - 1f, world.rectLeft.height);
        shapeRenderer.end();
    }

    public void resize(int width, int height) {
        viewport.update(width, height);
        viewport.apply();
        cam.position.set(cam.viewportWidth / 2f, cam.viewportHeight / 2f, 0f);
    }
}
