package no.ntnu.master.objects;

import com.badlogic.gdx.math.Vector2;

import java.security.SecureRandom;
import java.util.List;

import no.ntnu.master.utils.Enums.Direction;
import no.ntnu.master.utils.Settings;

public class Dot {

    private Direction direction;
    private boolean isRandom;
    private float radius;

    private float directionTimer;
    private float aliveTimer;
    private float maxAliveTime;

    private float horMaxTime;
    private float ranMaxTime;

    private Vector2 position;
    private Vector2 velocity;

    public boolean changeFlag = false;

    private float speed;

    private SecureRandom random;

    public Dot(float x, float y, float radius, Direction dir, float stayingAliveTime) {
        this.position = new Vector2(x, y);
        this.radius = radius;
        this.direction = dir;

        aliveTimer = stayingAliveTime;
        maxAliveTime = stayingAliveTime;

        random = new SecureRandom();
        horMaxTime = Settings.getDotHorizontalReversalTime();
        ranMaxTime = Settings.getDotRandomDirectionTime();
        speed = Settings.getDotVelocity();

        if (direction == Direction.LEFT) {
            velocity = new Vector2(-speed, 0f);
        } else {
            velocity = new Vector2(speed, 0f);
        }
        if (direction == Direction.RANDOM) {
            velocity.rotate(random.nextFloat() * (360f - 0f) + 0f);
            directionTimer = random.nextFloat() * (ranMaxTime - 0f) + 0f;
            isRandom = true;
        } else {
            directionTimer = 0f;
            isRandom = false;
        }
    }

    public void update(float delta) {
        directionTimer += delta;
        aliveTimer -= delta;

        if (!isRandom && directionTimer >= horMaxTime) {
            velocity.rotate(180f);
            directionTimer = 0f;
        }

        if (isRandom && directionTimer >= ranMaxTime) {
            velocity.rotate(random.nextFloat() * (360f - 0f) + 0f);
            directionTimer = 0f;
        }

        position.add(velocity.cpy().scl(delta));
    }



    public void wallCollision(boolean verticalWall, float delta) {
        if (verticalWall)
            velocity.x = -(velocity.x);
        else
            velocity.y = -(velocity.y);
        position.add(velocity.cpy().scl(delta));
    }

    public void detectDotCollision(List<Dot> dots, float delta) {
        for (int i = 0; i < dots.size(); i++) {
            Dot dot = dots.get(i);
            if (dot != this) {
                if (this.getPosition().dst(dot.getPosition()) <= (this.radius + dot.getRadius())) {
                    calculateNewVelocity(dot, delta);
                }
            }
         }
    }

    private void calculateNewVelocity(Dot dot, float delta) {
        float mass1 = 1f;
        float mass2 = 1f;

//        float collisionX = ((this.getX() * dot.getRadius()) + (dot.getX() * this.radius))
//                            / (this.radius + dot.radius);
//        float collisionY = ((this.getY() * dot.getRadius()) + (dot.getY() * this.radius))
//                / (this.radius + dot.radius);

        float velX1 = this.velocity.x;
        float velY1 = this.velocity.y;
        float velX2 = dot.velocity.x;
        float velY2 = dot.velocity.y;

        float newVelX1 = (velX1 * (mass1 - mass2) + (2 * mass2 * velX2)) / (mass1 + mass2);
        float newVelY1 = (velY1 * (mass1 - mass2) + (2 * mass2 * velY2)) / (mass1 + mass2);
        float newVelX2 = (velX2 * (mass2 - mass1) + (2 * mass1 * velX1)) / (mass1 + mass2);
        float newVelY2 = (velY2 * (mass2 - mass1) + (2 * mass1 * velY1)) / (mass1 + mass2);

        if (isRandom && !dot.isRandom) {
            velocity.set(newVelX1, newVelY2);
            position.add(velocity.cpy().scl(delta));
        } else if (!isRandom && dot.isRandom) {
            dot.velocity.rotate(180);
            dot.position.add(dot.velocity.cpy().scl(delta));
        } else if (isRandom && dot.isRandom) {
            velocity.set(newVelX1, newVelY1);
            dot.velocity.set(newVelX2, newVelY2);
            //if overlapping after update
            if (position.add(velocity.cpy().scl(delta)).dst(dot.position.add(dot.velocity.cpy().scl(delta))) < getRadius()+dot.getRadius()) {
                this.wallCollision(false, delta);
                this.wallCollision(true, delta);
            } else {
                position.add(velocity.cpy().scl(delta));
                dot.position.add(velocity.cpy().scl(delta));
            }
        }
    }

    public void setPosition(float x, float y) {
        position = new Vector2(x, y);
        changeFlag = true;
    }

    public Vector2 getPosition() {
        return position;
    }

    public float getX() {
        return position.x;
    }

    public float getY() {
        return position.y;
    }

    public float getRadius() {
        return radius;
    }

    public Vector2 getVelocity() {
        return velocity;
    }

    public void setVelocity(Vector2 velocity) {
        this.velocity = velocity;
    }

    public void setVelocity(float x, float y) {
        this.velocity = new Vector2(x, y);
    }

    public float getAliveTimer() {
        return aliveTimer;
    }

    public void setAliveTimer(float f) {
        this.aliveTimer = f;
    }

    public void resetAliveTimer() {
        this.aliveTimer = maxAliveTime;
        this.changeFlag = false;
    }

    public boolean isRandom() { return isRandom; }
}