package no.ntnu.master.objects;

import com.badlogic.gdx.Input;
import com.badlogic.gdx.utils.Array;

import no.ntnu.master.utils.Psychophysics;
import no.ntnu.master.utils.Settings;

/**
 * Created by B on 27.01.2016.
 * Used to create an extended results report with settings and details of reversal points.
 * 
 */
public class Results {
    private int correctAnswers;
    private int wrongAnswers;

    private float threshold;
    private float lowestCoherency = 100;

    private Array<String> reversalValues = new Array<>();
    private Array<String> screenSettings = new Array<>();
    private Array<String> motionSettings = new Array<>();
    private Array<String> formSettings = new Array<>();
    private Array<String> staircaseSettings = new Array<>();
    private Array<String> inputSettings = new Array<>();

    public Results(Array<Float> reversalPoints, int correctAnswers, int wrongAnswers) {
        this.correctAnswers = correctAnswers;
        this.wrongAnswers = wrongAnswers;

        threshold = Psychophysics.geometricMean(
                reversalPoints, Settings.getStaircaseReversalsToUse());

        for (int i=0;i<reversalPoints.size;i++) {
            if (reversalPoints.get(i) < lowestCoherency) {
                lowestCoherency = reversalPoints.get(i);
            }
            reversalValues.add((i+1) + ": " + reversalPoints.get(i));
        }

        screenSettings.add("screen_w_mm: " + Settings.getScreenWidth());
        screenSettings.add("screen_h_mm: " + Settings.getScreenHeight());
        screenSettings.add("screen_w_px: " + Settings.getScreenWidthPixels());
        screenSettings.add("screen_h_px: " + Settings.getScreenHeightPixels());
        screenSettings.add("viewing_distance: " + Settings.getScreenViewingDistance());
        screenSettings.add("patch_width: "+Settings.getPatchWidth());
        screenSettings.add("patch_height: "+Settings.getPatchHeight());
        screenSettings.add("patch_gap: " + Settings.getPatchGap());

        motionSettings.add("dot_amount: " + Settings.getDotMaxAmount());
        motionSettings.add("dot_radius: " + Settings.getDotRadiusPixels());
        motionSettings.add("dot_spacing: " + Settings.getDotSpacing());
        motionSettings.add("dot_velocity: " + Settings.getDotVelocity());
        motionSettings.add("dot_coherency: " + Settings.getDotCoherency());
        motionSettings.add("dot_animation_time: " + Settings.getDotAnimationTime());
        motionSettings.add("dot_max_life_time: " + Settings.getDotMaxLifeTime());
        motionSettings.add("dot_kill_percentage: " + Settings.getDotPercentageToKill());
        motionSettings.add("dot_horizontal_reversal_time: " + Settings.getDotHorizontalReversalTime());
        motionSettings.add("dot_random_direction_time: " + Settings.getDotRandomDirectionTime());

        formSettings.addAll("form_auto_mode: " + Settings.getFormAutoMode());
        formSettings.add("form_line_amount: " + Settings.getFormMaxLineSegments());
        formSettings.add("form_diameter_wb: " + Settings.getFormDiameterWb());
        formSettings.add("form_nr_of_circles: " + Settings.getFormNrOfCircles());
        formSettings.add("form_circle_gap: " + Settings.getFormCirclesGap());
        formSettings.add("form_line_length: " + Settings.getFormLineLength());
        formSettings.add("form_line_height: " + Settings.getFormLineHeight());
        formSettings.add("form_line_gap: " + Settings.getFormLineGap());
        formSettings.add("form_coherency: " + Settings.getFormCoherency());
        formSettings.add("form_f_detection_time: " + Settings.getFormFDetectionTime());
        formSettings.add("form_r_detection_time: " + Settings.getFormRDetectionTime());

        staircaseSettings.add("stair_correct_db: " + Settings.getStaircaseCorrectDB());
        staircaseSettings.add("stair_wrong_db: " + Settings.getStaircaseWrongDB());
        staircaseSettings.add("stair_max_tries: " + Settings.getStaircaseMaxTries());
        staircaseSettings.add("stair_reversal_points: " + Settings.getStaircaseReversalPoints());
        staircaseSettings.add("stair_mean_from_last: " + Settings.getStaircaseReversalsToUse());

        inputSettings.addAll("input_key_left: " + Input.Keys.toString(Settings.getInputKeyLeft()));
        inputSettings.addAll("input_key_right: " + Input.Keys.toString(Settings.getInputKeyRight()));
        inputSettings.addAll("input_key_back: " + Input.Keys.toString(Settings.getInputKeyBack()));
        inputSettings.addAll("input_continuous_mode: " + Settings.getInputContinuousMode());

    }

    public float getThreshold() {
        return threshold;
    }

    public float getLowestCoherency() {
        return lowestCoherency;
    }
 }
