package no.ntnu.master.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.ScreenAdapter;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.TextArea;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import com.badlogic.gdx.utils.Array;
import com.badlogic.gdx.utils.Json;
import com.badlogic.gdx.utils.viewport.ScreenViewport;

import no.ntnu.master.DyslexiaDetection;
import no.ntnu.master.menus.MainMenu;
import no.ntnu.master.objects.Results;
import no.ntnu.master.utils.AssetLoader;
import no.ntnu.master.utils.Settings;


/**
 * Created by B on 22.01.2016.
 * Screen for showing the results after a test is finished.
 */
public class ResultsScreen extends ScreenAdapter {
    private DyslexiaDetection game;
    private Array<Float> reversalValues;

    private Skin uiSkin = AssetLoader.uiSkin;

    private OrthographicCamera cam;
    private ScreenViewport viewport;

    private Stage stage;
    private Table paneTable;
    private ScrollPane scrollPane;

    private TextButton mainMenu, restart, shareResults;

    private TextArea textArea;

    private Label screenLabel;
    private Label thresholdLabel;
    private Label lowestCoherencyLabel;


    private Results results;
    private Json json;

    private int correctAnswers;
    private int wrongAnswers;

    private final String SCREEN_NAME_TEXT = "Test results";
    private final String REVERSAL_FIELD_TEXT = "";
    private final String MAIN_MENU_TEXT = "Main menu";
    private final String RESTART_TEXT = "Restart test";
    private final String SHARE_RESULTS_TEXT = "Copy extended results to clipboard";

    public ResultsScreen(DyslexiaDetection game, Results results) {
        this.game = game;
        this.results = results;
        json = new Json();

        initScreen();
        initUiElements();
        buildUi();
    }

    /**
     * Creates a camera and sets the viewport for the camera and stage
     */
    private void initScreen() {
        //setup a camera to show the screen
        cam = new OrthographicCamera();
        viewport = new ScreenViewport(cam);
        viewport.apply();
        cam.position.set(cam.viewportWidth / 2, cam.viewportHeight / 2, 0);

        //stage and table for ui elements
        stage = new Stage(viewport, game.batch);
        paneTable = new Table();
        scrollPane = new ScrollPane(paneTable, uiSkin);

        //Use the stage for input processing
        Gdx.input.setInputProcessor(stage);
    }

    /**
     * Create and fill the stage with the gui elements
     */
    private void initUiElements() {
        screenLabel = new Label(SCREEN_NAME_TEXT, uiSkin);

        thresholdLabel = new Label("Threshold: " + results.getThreshold(),uiSkin);
//        lowestCoherencyLabel = new Label("Lowest coherency: " + results.getLowestCoherency(),uiSkin);

//        textArea = new TextArea(json.prettyPrint(results), uiSkin);
//        textArea.setTouchable(Touchable.disabled);
//        textArea.setPrefRows(60);

        mainMenu = new TextButton(MAIN_MENU_TEXT, uiSkin);
        mainMenu.addListener(new ChangeListener() {
            @Override
            public void changed(ChangeEvent event, Actor actor) {
                game.setScreen(new MainMenu(game));
            }
        });
        mainMenu.setColor(Color.ORANGE);

        shareResults = new TextButton(SHARE_RESULTS_TEXT, uiSkin);
        shareResults.addListener(new ChangeListener() {
            @Override
            public void changed(ChangeEvent event, Actor actor) {
                Gdx.app.getClipboard().setContents(json.prettyPrint(results));
            }
        });
        shareResults.setColor(Color.LIGHT_GRAY);
    }

    /**
     * Populate the table with ui elements
     */
    private void buildUi() {
        scrollPane.setFillParent(true);

        paneTable.add(screenLabel).pad(20f).row();

        paneTable.add(thresholdLabel).pad(5f).row();
        paneTable.add(lowestCoherencyLabel).pad(5f).padBottom(40f).row();



//        paneTable.add(textArea).width(viewport.getScreenWidth()/2).row();

        paneTable.add(mainMenu).colspan(2).fill().height(viewport.getScreenHeight() / 10f).pad(5f).row();
        paneTable.add(shareResults).colspan(2).fill().height(viewport.getScreenHeight() / 10f).pad(5f).row();

        stage.addActor(scrollPane);
    }
    /**
     * Render the screen
     * @param delta delta time
     */
    @Override
    public void render(float delta) {
        Gdx.gl.glClearColor(0f, 0f, 0f, 1f);
        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
        if (Gdx.input.isKeyPressed(Input.Keys.BACK) || Gdx.input.isKeyJustPressed(Settings.getInputKeyBack())) {
            game.backPushed = true;
            game.setScreen(new MainMenu(game));
        }
        stage.act(delta);
        stage.draw();
    }

    /**
     * Called when the window is resized
     * @param width the new window width
     * @param height the new window height
     */
    @Override
    public void resize(int width, int height) {
        viewport.update(width, height);
        cam.position.set(cam.viewportWidth / 2, cam.viewportHeight / 2, 0);
    }

    @Override
    public void pause() {

    }

    @Override
    public void resume() {

    }

    @Override
    public void hide() {

    }

    @Override
    public void dispose() {
        uiSkin.dispose();
    }
}
