package no.ntnu.master.utils;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.graphics.g2d.freetype.FreeTypeFontGenerator;
import com.badlogic.gdx.graphics.g2d.freetype.FreeTypeFontGenerator.FreeTypeFontParameter;
import com.badlogic.gdx.scenes.scene2d.ui.Dialog;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.ui.TextField;
import com.badlogic.gdx.scenes.scene2d.ui.Window;

/**
 * Created by B on 02.07.2015.
 * Class for loading assets used in the application
 */
public class AssetLoader {

    public static Texture dotTexture, lineTexture;

    //add the fonts
    public static BitmapFont font;

    public static BitmapFont shadow;

    private static FreeTypeFontGenerator generator;
    private static FreeTypeFontParameter parameter;

    //add free ui from http://www.microbasic.net/2014/05/free-cc0-ui-elements-to-be-used-in-your-libgdx-games/
    private static TextureAtlas whiteUi;
    private static TextureAtlas commonsUi;
    public static Skin uiSkin;

    public static void load() {


        dotTexture = new Texture(Gdx.files.internal("img/dot.png"));
        dotTexture.setFilter(Texture.TextureFilter.Nearest, Texture.TextureFilter.Nearest);

        lineTexture = new Texture(Gdx.files.internal("img/line.png"));
        lineTexture.setFilter(Texture.TextureFilter.Nearest, Texture.TextureFilter.Nearest);

        //load the fonts
        int fSize = (int)Math.ceil(Gdx.graphics.getDesktopDisplayMode().width * 32 / 1280);
        generator = new FreeTypeFontGenerator(Gdx.files.internal("font/arial.ttf"));
        parameter = new FreeTypeFontParameter();
        parameter.size = fSize;
        font = new BitmapFont(Gdx.files.internal("font/font.fnt"), false);
        font.getRegion().getTexture().setFilter(Texture.TextureFilter.Linear, Texture.TextureFilter.Linear);
        shadow = generator.generateFont(parameter);
        shadow.getRegion().getTexture().setFilter(Texture.TextureFilter.Linear, Texture.TextureFilter.Linear);
        generator.dispose();

        whiteUi = new TextureAtlas(Gdx.files.internal("atlas/ui-white.atlas"));
        commonsUi = new TextureAtlas(Gdx.files.internal("atlas/ui-commons.atlas"));
        uiSkin = new Skin();

        //load free uielements into the skin
        uiSkin.addRegions(whiteUi);
        uiSkin.addRegions(commonsUi);
//        final Slider.SliderStyle st = new Slider.SliderStyle();
//        st.background = uiSkin.getDrawable("slider_back_hor");
//        st.knob = uiSkin.getDrawable("knob_01");
//        uiSkin.add("default-horizontal", st);
        
        final Label.LabelStyle lt = new Label.LabelStyle();
        lt.font = shadow;
        //lt.fontColor = Color.WHITE;
        uiSkin.add("default", lt);

        final TextField.TextFieldStyle tt = new TextField.TextFieldStyle();
        tt.background = uiSkin.getDrawable("textbox_01");
        tt.disabledBackground = uiSkin.getDrawable("button_01");
        tt.disabledFontColor = Color.LIGHT_GRAY;
        tt.cursor = uiSkin.getDrawable("textbox_cursor_02");
        tt.font = shadow;
        tt.fontColor = Color.BLACK;
        tt.selection = uiSkin.newDrawable("textbox_01", 0.5f, 0.5f, 0.5f, 0.5f);
        uiSkin.add("default", tt);

        final TextButton.TextButtonStyle bt = new TextButton.TextButtonStyle();
        bt.up = uiSkin.getDrawable("button_01");
        bt.down = uiSkin.getDrawable("button_02");
        bt.font = shadow;
        bt.downFontColor = Color.LIGHT_GRAY;
        bt.pressedOffsetY = -3f;
        bt.fontColor = Color.BLACK;
        uiSkin.add("default", bt);

        final Dialog.WindowStyle dt = new Window.WindowStyle();
        dt.background = uiSkin.getDrawable("window_02");
        dt.background.setTopHeight(dt.background.getTopHeight() + shadow.getCapHeight());
        dt.stageBackground = uiSkin.getDrawable("transparent-black-70");
        dt.titleFont = shadow;
        dt.titleFontColor = Color.BLACK;
        uiSkin.add("default", dt);

        final ScrollPane.ScrollPaneStyle sp = new ScrollPane.ScrollPaneStyle();
        sp.hScroll = uiSkin.getDrawable("scroll_back_hor");
        sp.hScrollKnob = uiSkin.getDrawable("knob_05");
        sp.vScroll = uiSkin.getDrawable("scroll_back_ver");
        sp.vScrollKnob = uiSkin.getDrawable("knob_06");
        uiSkin.add("default", sp);

        final CheckBoxReversed.CheckBoxStyle cp = new CheckBoxReversed.CheckBoxStyle();
        cp.font = shadow;
        cp.checkboxOn = uiSkin.getDrawable("checkbox_on");
        cp.checkboxOff = uiSkin.getDrawable("checkbox_off");
        uiSkin.add("default", cp);
    }

    public static void dispose() {
        dotTexture.dispose();
        lineTexture.dispose();
        font.dispose();
        shadow.dispose();
        uiSkin.dispose();
    }
}
