package no.ntnu.master.utils;

import com.badlogic.gdx.InputProcessor;
import com.badlogic.gdx.utils.Array;

import no.ntnu.master.DyslexiaDetection;
import no.ntnu.master.menus.MainMenu;
import no.ntnu.master.objects.Results;
import no.ntnu.master.screens.ResultsScreen;
import no.ntnu.master.utils.Enums.WorldStates;

/**
 * Created by B on 02.07.2015.
 * Class for registering input
 */
public class InputHandler implements InputProcessor {

    private DyslexiaDetection game;
    private World world;

    private int maxSteps;
    private int stepCounter;
    private int reversalPoints;

    //TODO make non static before finished version
    private static int reversalCounter;
    private int correctAnswerCounter;
    private int wrongAnswerCounter;

    private boolean prevStep;
    private Array<Float> reversalValues;

    private float factorCorrectAnswer;
    private float factorWrongAnswer;

    /**
     * Constructor taking a game and world object
     * @param game reference to the game object
     * @param world reference to a MotionWorld object
     */
    public InputHandler(DyslexiaDetection game, World world) {
        this.game = game;
        this.world = world;

        maxSteps = Settings.getStaircaseMaxTries();
        reversalPoints = Settings.getStaircaseReversalPoints();
        stepCounter = 0;

        reversalCounter = 0;
        wrongAnswerCounter = 0;
        correctAnswerCounter = 0;

        reversalValues = new Array<>();

        factorCorrectAnswer = Psychophysics.decibelToFactor(Settings.getStaircaseCorrectDB());
        factorWrongAnswer = Psychophysics.decibelToFactor(Settings.getStaircaseWrongDB());
    }

    @Override
    public boolean keyDown(int keycode) {
        return false;
    }

    @Override
    public boolean keyUp(int keycode) {
        if (keycode == Settings.getInputKeyLeft()) {
            touchDown((Settings.getScreenWidthPixels() / 2)-10, 0, 0, 0);
        }
        else if (keycode == Settings.getInputKeyRight()) {
            touchDown((Settings.getScreenWidthPixels() / 2)+10, 0, 0, 0);
        }
        else if (keycode == Settings.getInputKeyBack()) {
            game.setScreen(new MainMenu(game));
        }
        return false;
    }

    @Override
    public boolean keyTyped(char character) {
        return false;
    }

    @Override
    public boolean touchDown(int screenX, int screenY, int pointer, int button) {
//        Gdx.app.log("UITest", "screenX: " + screenX + " screenY: " + (Settings.getScreenHeightPixels() - screenY) + " pointer: " + pointer + " button: " + button);

        if (world.getState() == WorldStates.FINISHED) {
            game.setScreen(new ResultsScreen(game, new Results(reversalValues, correctAnswerCounter, wrongAnswerCounter)));
            //game.setScreen(new ResultsScreen(game, reversalValues, correctAnswerCounter, wrongAnswerCounter));
        }


        //Check if the world is in pause, otherwise ignore input
        if (world.getState() == WorldStates.PAUSED || !world.getIsFixed() || Settings.getInputContinuousMode()) {
            if (stepCounter > 0) {
                check(screenX, screenY);
                stepCounter++;
            } else stepCounter++;
            if (stepCounter-1 == maxSteps || reversalCounter == reversalPoints) {
                world.setState(WorldStates.FINISHED);
            } else world.setState(WorldStates.RUNNING);

        }

        return false;
    }

    private void check(int screenX, int screenY) {
        String coherentSide = world.getCoherentPatch();
        float reversalValue = world.getCoherencyPercent();
        boolean thisStep = true;

        if (coherentSide.equals("Right") && screenX > Settings.getScreenWidthPixels() / 2
                || coherentSide.equals("Left") && screenX < Settings.getScreenWidthPixels() / 2) {
            correctAnswerCounter++;
            world.updateCoherency(factorCorrectAnswer, true);
        } else  {
            wrongAnswerCounter++;
            world.updateCoherency(factorWrongAnswer, false);
            thisStep = false;
        }

        if (stepCounter > 1 && prevStep != thisStep) {
            reversalValues.add(reversalValue);
            reversalCounter++;
        }
        prevStep = thisStep;
    }

    @Override
    public boolean touchUp(int screenX, int screenY, int pointer, int button) {
        return false;
    }

    @Override
    public boolean touchDragged(int screenX, int screenY, int pointer) {
        return true;
    }

    @Override
    public boolean mouseMoved(int screenX, int screenY) {
        return false;
    }

    @Override
    public boolean scrolled(int amount) {
        return false;
    }

    static public int getReversalCounter() {
        return reversalCounter;
    }
}
